#-----------------------------------------------------------------#
#    SCRIPT: Load_CharacterMarkerSetMapping.py
#
#    DESCRIPTION: sample code for MarkingSet Mapping
#
# 1. Copy <machine name>.<chara name>_Mapping.txt to 
#     C:\Users\<username>\Documents\MB\2014-x64\config
# 2. Load Motion File from File > Motion File Import.
#     Flock of markers will appear.
# 3. Load Characterized Character.
# 4. Adjust SRT of parent of markers, and fit to Character.
# 5. Execute this script.
# 6. Playback the scene.
#-----------------------------------------------------------------#

from pyfbsdk import *

def CharacterMapping(pSave, pFile = None, pCharacter = FBApplication().CurrentCharacter):
    if pCharacter == None:
        return
    
    if pFile == None:
        pFile = "@%s_Mapping.txt" % pCharacter.Name

    lMarkerSet = pCharacter.GetCharacterMarkerSet(True)
    
    lConfigFile = FBConfigFile(pFile)
    if pSave:
        if lMarkerSet:
            lConfigFile.ClearFile()
            for lProp in lMarkerSet.PropertyList:
                if lProp.Name.endswith('.Markers'):
                    
                    lConstraintType = lMarkerSet.PropertyList.Find(lProp.Name.replace('.Markers', '.Constraint'))
                    if lConstraintType != None:
                        lConfigFile.Set(lProp.Name, "Type", "%d" % lConstraintType.Data)
                    else:
                        continue
                        
                    for lSrcId in range(lProp.GetSrcCount()):
                        lConfigFile.Set(lProp.Name, "Marker%d" % lSrcId, lProp.GetSrc(lSrcId).LongName)
    else:
        if lMarkerSet:
            lMarkerSet.FBDelete()
        
        pCharacter.CreateCharacterMarkerSet(True)
        lMarkerSet = pCharacter.GetCharacterMarkerSet(True)
        
        FBBeginChangeAllModels()
        
        for lProp in lMarkerSet.PropertyList:
            if lProp.Name.endswith('.Markers'):               
                lMarkerList = []
                lMarkerIndex = 0
                while True:
                    lMarkerName = lConfigFile.Get(lProp.Name, "Marker%d" % lMarkerIndex)
                    if lMarkerName:
                        lMarkerIndex += 1
                        lMarkerModel = FBFindModelByLabelName(lMarkerName)
                        if lMarkerModel:
                            lMarkerList.append(lMarkerModel)
                    else:
                        break
                        
                lProp.BeginChange()
                lProp.DisconnectAllSrc()
                
                for lMarker in lMarkerList:
                    lProp.ConnectSrc(lMarker)
                    #ConnectSrcIfMissing(lProp,lMarker)
                
                lProp.EndChange()

                lConstraintType = lMarkerSet.PropertyList.Find(lProp.Name.replace('.Markers', '.Constraint'))
                if lConstraintType != None:
                    lType = lConfigFile.Get(lProp.Name, "Type")
                    if lType:
                        lConstraintType.Data = int(lType)
                               
        FBEndChangeAllModels()

def LoadMapping(pFile = None, pCharacter = FBApplication().CurrentCharacter):
    CharacterMapping(False,pFile,pCharacter)
    
def SaveMapping(pFile = None, pCharacter = FBApplication().CurrentCharacter):
    CharacterMapping(True,pFile,pCharacter)
    
SaveMapping()